Class: RDF::Query

Inherits:
Object
  • Object
show all
Includes:
Enumerable
Defined in:
lib/rdf/query.rb,
lib/rdf/query/pattern.rb,
lib/rdf/query/solution.rb,
lib/rdf/query/variable.rb,
lib/rdf/query/solutions.rb,
lib/rdf/query/hash_pattern_normalizer.rb

Overview

An RDF basic graph pattern (BGP) query.

Named queries either match against a specifically named graph if the name is an RDF::Resource or bound RDF::Query::Variable. Names that are against unbound variables match either default or named graphs. The name of false will only match against the default graph.

Variable names cause the variable to be added to the solution set elements.

Examples:

Constructing a basic graph pattern query (1)

query = RDF::Query.new do
  pattern [:person, RDF.type,  FOAF.Person]
  pattern [:person, FOAF.name, :name]
  pattern [:person, FOAF.mbox, :email]
end

Constructing a basic graph pattern query (2)

query = RDF::Query.new({
  person: {
    RDF.type  => FOAF.Person,
    FOAF.name => :name,
    FOAF.mbox => :email,
  }
})

Executing a basic graph pattern query

graph = RDF::Graph.load('etc/doap.nt')
query.execute(graph).each do |solution|
  puts solution.inspect
end

Constructing and executing a query in one go (1)

solutions = RDF::Query.execute(graph) do
  pattern [:person, RDF.type, FOAF.Person]
end

Constructing and executing a query in one go (2)

solutions = RDF::Query.execute(graph, {
  person: {
    RDF.type => FOAF.Person,
  }
})

In this example, the default graph contains the names of the publishers of two named graphs. The triples in the named graphs are not visible in the default graph in this example.

# default graph
@prefix dc: <http://purl.org/dc/elements/1.1/

<http://example.org/bob>    dc:publisher  "Bob" .
<http://example.org/alice>  dc:publisher  "Alice" .

# Named graph: http://example.org/bob
@prefix foaf: <http://xmlns.com/foaf/0.1/> .

_:a foaf:name "Bob" .
_:a foaf:mbox <mailto:bob@oldcorp.example.org> .

# Named graph: http://example.org/alice
@prefix foaf: <http://xmlns.com/foaf/0.1/> .

_:a foaf:name "Alice" .
_:a foaf:mbox <mailto:alice@work.example.org> .

See Also:

Since:

  • 0.3.0

Defined Under Namespace

Classes: HashPatternNormalizer, Pattern, Solution, Solutions, Variable

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Enumerable

#dump, #each_graph, #each_object, #each_predicate, #each_quad, #each_subject, #each_term, #each_triple, #enum_graph, #enum_object, #enum_predicate, #enum_quad, #enum_statement, #enum_subject, #enum_term, #enum_triple, #graph_names, #has_graph?, #has_object?, #has_predicate?, #has_quad?, #has_statement?, #has_subject?, #has_term?, #has_triple?, #invalid?, #objects, #predicates, #project_graph, #quads, #statements, #subjects, #supports?, #terms, #to_a, #to_h, #to_set, #triples

Methods included from Util::Aliasing::LateBound

#alias_method

Methods included from Countable

#count, #enum_for

Constructor Details

#initialize(patterns = [], **options) {|query| ... } ⇒ Query #initialize(patterns, **options) ⇒ Query

Initializes a new basic graph pattern query.

Overloads:

  • #initialize(patterns = [], **options) {|query| ... } ⇒ Query

    Parameters:

    • patterns (Array<RDF::Query::Pattern>) (defaults to: [])

    • options (Hash{Symbol => Object})

      any additional keyword options

    Options Hash (**options):

    • :solutions (RDF::Query::Solutions) — default: Solutions.new
    • :graph_name (RDF::Resource, RDF::Query::Variable, false) — default: nil

      Default graph name for matching against queryable. Named queries either match against a specifically named graphs if the name is an Resource or bound Variable. Names that are against unbound variables match either default or named graphs. The name of false will only match against the default graph.

    • :name (RDF::Resource, RDF::Query::Variable, false) — default: nil

      Alias for :graph_name.

    Yields:

    • (query)

    Yield Parameters:

    Yield Returns:

    • (void)

      ignored

  • #initialize(patterns, **options) ⇒ Query

    Parameters:

    • patterns (Hash{Object => Object})

    • solutions (RDF::Query::Solutions)

      (Solutions.new)

    • graph_name (RDF::Resource, RDF::Query::Variable, false)

      (false) Default graph name for matching against queryable. Named queries either match against a specifically named graphs if the name is an Resource or bound Variable. Names that are against unbound variables match either default or named graphs. The name of false will only match against the default graph.

    • name (RDF::Resource, RDF::Query::Variable, false)

      (false) Alias for :graph_name.

    • options (Hash{Symbol => Object})

      any additional keyword options

Parameters:

  • options (Hash)

    a customizable set of options

Options Hash (**options):

  • validate (Boolean) — default: false

    validate patterns @yield [query] @yieldparam [RDF::Query] query @yieldreturn [void] ignored

Since:

  • 0.3.0



185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
# File 'lib/rdf/query.rb', line 185

def initialize(*patterns, solutions: nil, graph_name: nil, name: nil, validate: false, **options, &block)
  @options = options.dup
  @solutions = Query::Solutions(solutions)
  graph_name = name if graph_name.nil?
  @graph_name = graph_name

  patterns << @options if patterns.empty?

  @patterns  = case patterns.first
    when Hash  then compile_hash_patterns(HashPatternNormalizer.normalize!(patterns.first.dup, @options))
    when Array then patterns.first
    else patterns
  end

  if block_given?
    case block.arity
      when 1 then block.call(self)
      else instance_eval(&block)
    end
  end

  validate! if validate
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method in the class RDF::Enumerable

Instance Attribute Details

#graph_nameRDF::Resource, ...

Scope the query to named graphs matching value

Returns:

Since:

  • 0.3.0



141
142
143
# File 'lib/rdf/query.rb', line 141

def graph_name
  @graph_name
end

#optionsHash (readonly)

Any additional options for this query.

Returns:

  • (Hash)

Since:

  • 0.3.0



135
136
137
# File 'lib/rdf/query.rb', line 135

def options
  @options
end

#patternsArray<RDF::Query::Pattern> (readonly)

The patterns that constitute this query.

Returns:

Since:

  • 0.3.0



123
124
125
# File 'lib/rdf/query.rb', line 123

def patterns
  @patterns
end

#solutionsRDF::Query::Solutions (readonly)

The solution sequence for this query.

Returns:

Since:

  • 0.3.0



129
130
131
# File 'lib/rdf/query.rb', line 129

def solutions
  @solutions
end

Class Method Details

.execute(queryable, patterns = {}, options = {}) {|query| ... } ⇒ RDF::Query::Solutions

Executes a query on the given queryable graph or repository.

Parameters:

  • queryable (RDF::Queryable)

    the graph or repository to query

  • patterns (Hash{Object => Object}) (defaults to: {})

    optional hash patterns to initialize the query with

  • options (Hash{Symbol => Object}) (defaults to: {})

    any additional keyword options (see #initialize)

Yields:

  • (query)

Yield Parameters:

Yield Returns:

  • (void)

    ignored

Returns:

See Also:

Since:

  • 0.3.0



92
93
94
# File 'lib/rdf/query.rb', line 92

def self.execute(queryable, patterns = {}, options = {}, &block)
  self.new(patterns, **options, &block).execute(queryable, **options)
end

.SolutionsSolutions .Solutions(solutions) ⇒ Solutions .Solutions(array) ⇒ Solutions .Solutions(*args) ⇒ Solutions

Cast values as Solutions

Overloads:

  • .SolutionsSolutions

    Returns Solutions.new()

    Returns:

  • .Solutions(solutions) ⇒ Solutions

    Returns the argument

    Returns:

  • .Solutions(array) ⇒ Solutions

    Returns the array extended with solutions

    Parameters:

    • array (Array)

    Returns:

    • (Solutions)

      returns the array extended with solutions

  • .Solutions(*args) ⇒ Solutions

    Returns new solutions including the arguments, which must each be a Solution

    Parameters:

    Returns:

    • (Solutions)

      returns new solutions including the arguments, which must each be a Solution

Since:

  • 0.3.0



111
112
113
114
115
116
117
# File 'lib/rdf/query.rb', line 111

def self.Solutions(*args)
  if args.length == 1
    return args[0] if args[0].is_a?(Solutions)
    args = args[0] if args[0].is_a?(Array)
  end
  return Solutions.new(args)
end

Instance Method Details

#+(other) ⇒ RDF::Query

Add patterns from another query to form a new Query

Parameters:

Returns:

Since:

  • 0.3.0



408
409
410
# File 'lib/rdf/query.rb', line 408

def +(other)
  Query.new(self.patterns + other.patterns)
end

#<<(pattern) ⇒ void

This method returns an undefined value.

Appends the given query pattern to this query.

Parameters:

Since:

  • 0.3.0



215
216
217
218
# File 'lib/rdf/query.rb', line 215

def <<(pattern)
  @patterns << Pattern.from(pattern)
  self
end

#apply_graph_name(graph_name = nil) ⇒ Object

Apply the graph name specified (or configured) to all patterns that have no graph name

Parameters:

Since:

  • 0.3.0



433
434
435
436
# File 'lib/rdf/query.rb', line 433

def apply_graph_name(graph_name = nil)
  graph_name ||= self.graph_name
  patterns.each {|pattern| pattern.graph_name = graph_name if pattern.graph_name.nil?} unless graph_name.nil?
end

#default?Boolean

Is this query scoped to the default graph?

Returns:

  • (Boolean)

Since:

  • 0.3.0



420
421
422
# File 'lib/rdf/query.rb', line 420

def default?
  graph_name == false
end

#dupRDF::Query

Duplicate query, including patterns and solutions

Returns:

Since:

  • 0.3.0



509
510
511
512
# File 'lib/rdf/query.rb', line 509

def dup
  patterns = @patterns.map {|p| p.dup}
  Query.new(patterns, graph_name: graph_name, solutions: @solutions.dup, **options)
end

#each_solution {|solution| ... } ⇒ Enumerator Also known as: each

Enumerates over each matching query solution.

Yields:

  • (solution)

Yield Parameters:

Returns:

Since:

  • 0.3.0



490
491
492
# File 'lib/rdf/query.rb', line 490

def each_solution(&block)
  @solutions.each(&block)
end

#each_statement {|RDF::Query::Pattern| ... } ⇒ Enumerator

Enumerates over each statement (pattern).

Yields:

Yield Parameters:

  • pattern (::Query::Pattern)

Returns:

Since:

  • 0.3.0



501
502
503
504
# File 'lib/rdf/query.rb', line 501

def each_statement(&block)
  apply_graph_name
  patterns.each(&block)
end

#empty?Boolean

Query has no patterns

Returns:

  • (Boolean)

Since:

  • 0.3.0



479
480
481
# File 'lib/rdf/query.rb', line 479

def empty?
  patterns.empty?
end

#execute(queryable, bindings: {}, solutions: Solution.new, graph_name: nil, name: nil, **options) {|solution| ... } ⇒ RDF::Query::Solutions

Note:

solutions could be an Iterator, but this algorithm cycles over solutions, which requires them to be an array internally.

Executes this query on the given queryable graph or repository.

Named queries either match against a specifically named graphs if the name is an RDF::Resource or bound RDF::Query::Variable. Names that are against unbound variables match either detault or named graphs. The name of false will only match against the default graph.

If the query nas no patterns, it returns a single empty solution as per SPARQL 1.1 _Empty Group Pattern_.

Parameters:

  • queryable (RDF::Queryable)

    the graph or repository to query

  • solutions (RDF::Query::Solutions) (defaults to: Solution.new)

    (Solutions.new)

  • graph_name (RDF::Resource, RDF::Query::Variable, false) (defaults to: nil)

    (nil) Default graph name for matching against queryable. Named queries either match against a specifically named graphs if the name is an Resource or bound Variable. Names that are against unbound variables match either default or named graphs. The name of false will only match against the default graph.

  • name (RDF::Resource, RDF::Query::Variable, false) (defaults to: nil)

    (nil) Alias for :graph_name.

  • options (Hash{Symbol => Object})

    any additional keyword options

Options Hash (**options):

  • bindings (Hash{Symbol => RDF::Term})

    optional variable bindings to use

  • :optimize (Boolean)

    Optimize query before execution.

  • solutions (RDF::Query::Solutions)

    optional initial solutions for chained queries

Yields:

  • (solution)

    each matching solution

Yield Parameters:

Yield Returns:

  • (void)

    ignored

Returns:

See Also:

Since:

  • 0.3.0



304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
# File 'lib/rdf/query.rb', line 304

def execute(queryable, bindings: {}, solutions: Solution.new, graph_name: nil, name: nil, **options, &block)
  # Use provided solutions to allow for query chaining
  # Otherwise, a quick empty solution simplifies the logic below; no special case for
  # the first pattern
  @solutions = Query::Solutions(solutions)

  # If there are no patterns, just return the empty solution
  if empty?
    @solutions.each(&block) if block_given?
    return @solutions
  end

  self.optimize! if options[:optimize]
  patterns = @patterns
  graph_name = name if graph_name.nil?
  @graph_name = graph_name unless graph_name.nil?

  # Add graph_name to pattern, if necessary
  unless @graph_name.nil?
    if patterns.empty?
      patterns = [Pattern.new(nil, nil, nil, graph_name: @graph_name)]
    else
      apply_graph_name(@graph_name)
    end
  end

  patterns.each do |pattern|

    old_solutions, @solutions = @solutions, Query::Solutions()

    bindings.each_key do |variable|
      if pattern.variables.include?(variable)
        unbound_solutions, old_solutions = old_solutions, Query::Solutions()
        bindings[variable].each do |binding|
          unbound_solutions.each do |solution|
            old_solutions << solution.merge(variable => binding)
          end
        end
        bindings.delete(variable)
      end
    end

    old_solutions.each do |solution|
      found_match = false
      pattern.execute(queryable, solution) do |statement|
        found_match = true
        @solutions << solution.merge(pattern.solution(statement))
      end
      # If this pattern was optional, and we didn't find any matches,
      # just copy it over as-is.
      if !found_match && pattern.optional?
        @solutions << solution
      end
    end

    #puts "solutions after #{pattern} are #{@solutions.to_a.inspect}"

    # It's important to abort failed queries quickly because later patterns
    # that can have constraints are often broad without them.
    # We have no solutions at all:
    return @solutions if @solutions.empty?

    if !pattern.optional?
      # We have no solutions for variables we should have solutions for
      # (excludes non-distinguished variables):
      need_vars = pattern.variables.select {|k,v| v.distinguished?}.keys
      @solutions.each do |solution|
        break if need_vars.empty?
        need_vars -= solution.bindings.keys
      end
      return Query::Solutions() unless need_vars.empty?
    end
  end
  @solutions.each(&block) if block_given?
  @solutions
end

#failed?Boolean

Returns true if this query did not match when last executed.

When the solution sequence is empty, this method can be used to determine whether the query failed to match or not.

Returns:

  • (Boolean)

See Also:

Since:

  • 0.3.0



389
390
391
# File 'lib/rdf/query.rb', line 389

def failed?
  @solutions.empty?
end

#matched?Boolean

Returns true if this query matched when last executed.

When the solution sequence is empty, this method can be used to determine whether the query matched successfully or not.

Returns:

  • (Boolean)

See Also:

Since:

  • 0.3.0



401
402
403
# File 'lib/rdf/query.rb', line 401

def matched?
  !failed?
end

#named?Boolean

Is this query scoped to a named graph?

Returns:

  • (Boolean)

Since:

  • 0.3.0



414
415
416
# File 'lib/rdf/query.rb', line 414

def named?
  !!graph_name
end

#node?Boolean Also known as: has_blank_nodes?

Returns true if any pattern contains a blank node.

Returns:

  • (Boolean)

Since:

  • 2.0



473
474
475
# File 'lib/rdf/query.rb', line 473

def node?
  patterns.any?(&:node?) || graph_name && graph_name.node?
end

#optimize(**options) ⇒ RDF::Query

Returns an optimized copy of this query.

Parameters:

  • options (Hash{Symbol => Object})

    any additional options for optimization

Returns:

Since:

  • 0.3.0



242
243
244
# File 'lib/rdf/query.rb', line 242

def optimize(**options)
  self.dup.optimize!(**options)
end

#optimize!(**options) ⇒ self

Optimizes this query by reordering its constituent triple patterns according to their cost estimates.

Parameters:

  • options (Hash{Symbol => Object})

    any additional options for optimization

Returns:

  • (self)

See Also:

Since:

  • 0.3.0



255
256
257
258
259
260
# File 'lib/rdf/query.rb', line 255

def optimize!(**options)
  @patterns.sort! do |a, b|
    (a.cost || 0) <=> (b.cost || 0)
  end
  self
end

#pattern(pattern, **options) ⇒ void

This method returns an undefined value.

Appends the given query pattern to this query.

Parameters:

  • pattern (RDF::Query::Pattern)

    a triple query pattern

  • options (Hash{Symbol => Object})

    any additional keyword options

Options Hash (**options):

  • :optional (Boolean) — default: false

    whether this is an optional pattern

Since:

  • 0.3.0



230
231
232
233
# File 'lib/rdf/query.rb', line 230

def pattern(pattern, **options)
  @patterns << Pattern.from(pattern, **options)
  self
end

#unnamed?Boolean

Is this query unscoped? This indicates that it can return results from either a named graph or the default graph.

Returns:

  • (Boolean)

Since:

  • 0.3.0



427
428
429
# File 'lib/rdf/query.rb', line 427

def unnamed?
  graph_name.nil?
end

#valid?Boolean

Determine if the query containts valid patterns

Returns:

  • (Boolean)

    true or false

Since:

  • 0.3.9



519
520
521
522
523
# File 'lib/rdf/query.rb', line 519

def valid?
  !!validate! rescue raise false
rescue
  false
end

#validate!RDF::Query

Validate this query, making sure it can be executed by our query engine. This method is public so that it may be called by implementations of RDF::Queryable#query_execute that bypass our built-in query engine.

Returns:

Raises:

  • (ArgumentError)

    This query cannot be executed.

Since:

  • 0.3.0



532
533
534
535
536
537
538
539
540
541
542
543
544
# File 'lib/rdf/query.rb', line 532

def validate!
  # All patterns must be valid
  @patterns.each(&:validate!)

  # All optional patterns must appear after the regular patterns.
  if i = @patterns.find_index(&:optional?)
    unless @patterns[i..-1].all?(&:optional?)
      raise ArgumentError.new("Optional patterns must appear at end of query")
    end
  end

  self
end

#variable?Boolean Also known as: variables?, has_variables?

Returns true if any pattern contains a variable.

Returns:

  • (Boolean)

Since:

  • 0.3.0



442
443
444
# File 'lib/rdf/query.rb', line 442

def variable?
  !variables.empty?
end

#variable_countInteger

Returns the number of variables in this query.

Returns:

  • (Integer)

    (0..3)

Since:

  • 0.3.0



464
465
466
# File 'lib/rdf/query.rb', line 464

def variable_count
  variables.keys.length
end

#variablesHash{Symbol => RDF::Query::Variable}

The variables used in this query. This includes variables used in patterns along with the graph_name itself, if it is a variable.

Returns:

Since:

  • 0.3.0



452
453
454
455
456
457
458
# File 'lib/rdf/query.rb', line 452

def variables
  # Set variables used in query
  vars = patterns.inject({}) do |memo, pattern|
    memo.merge(pattern.variables)
  end
  graph_name.is_a?(Variable) ? vars.merge(graph_name.to_sym => graph_name) : vars
end