Method: RDF::Query::Solutions#bindings

Defined in:
lib/rdf/query/solutions.rb

#bindingsHash{Symbol => Array<RDF::Term>}

Returns hash of bindings from each solution. Each bound variable will have an array of bound values representing those from each solution, where a given solution will have just a single value for each bound variable

Returns:

Since:

  • 0.3.0



99
100
101
102
103
104
105
106
107
108
# File 'lib/rdf/query/solutions.rb', line 99

def bindings
  bindings = {}
  each do |solution|
    solution.each do |key, value|
      bindings[key] ||= []
      bindings[key] << value
    end
  end
  bindings
end